<?php

/*
 * Discussion Board image uploader class
 * @link https://derekspringer.wordpress.com/2015/05/07/using-the-wordpress-media-loader-in-the-front-end/
*/

// Exit if accessed directly
if( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Plugin Uploader class
 **/

class CT_DB_Pro_Uploader {

	/*
	 * Initialize the class and start calling our hooks and filters
	 * @since 1.0.0
	 */
	public function init() {
		add_action( 'ctdb_start_new_topic_form', array( $this, 'change_caps' ) );
		add_filter( 'ctdb_topics_form', array( $this, 'filter_form' ) );
		add_filter( 'ctdb_post_id', array( $this, 'filter_post_id' ) );
		add_filter( 'upload_mimes', array( $this, 'custom_upload_mimes' ) );
		add_action( 'wp_enqueue_scripts', array( $this, 'enqueue_scripts' ) );
		add_filter( 'ajax_query_attachments_args', array( $this, 'filter_media_for_uploader' ) );
		add_filter( 'sanitize_file_name', array( $this, 'rename_upload' ), 10 );
	}

	/*
	 * Need to grant upload_files capability to permitted poster roles
	 * @since 1.1.0
	 */
	public function change_caps() {
		$permitted_roles = ctdb_get_permitted_poster_roles();
		if( ! empty( $permitted_roles ) ){
			foreach( $permitted_roles as $role ) {
				$role = get_role( $role );
				if( $role ) {
					$role->add_cap( 'upload_files' );
				}

			}
		}
	}

	/**
	 * Add upload button if user permitted to upload
	 * @since 1.0.0
	 * Specific to include_image_upload option
	 * @since 1.5.3
	 */
	public function filter_form( $form ) {
		if( $this -> is_uploading_permitted() && ! empty( $options['include_image_upload'] ) ) {
			// Add media upload button if enabled
			$form['fields']['upload_button'] = '<div><img style="margin: 1em 0;" id="frontend-image" /><input type="hidden" id="topic-image-id" name="topic-image-id"><input id="frontend-button" type="button" value="' . __( 'Upload Image', 'discussion-board-pro' ) . '" class="button" style="position: relative; z-index: 1;"><input id="frontend-remove-button" type="button" value="' . __( 'Remove Image', 'discussion-board-pro' ) . '" class="button" style="position: relative; z-index: 1; display: none;"></div>';
		}
		return $form;

	}

	public function enqueue_scripts() {
		if( $this -> is_uploading_permitted() ) {
			wp_enqueue_media();
		}
	}

	/**
	 * When a user is uploading an image, only allow the user to see their own uploaded
	 * images in the media gallery, not the entire media library.
	 *
	 * @since 1.8.10
	 *
	 * @param array $query The query options.
	 *
	 * @return array
	 */
	public function filter_media_for_uploader( $query ) {
		$user_id = get_current_user_id();

		if ( $user_id && ( ! current_user_can( 'activate_plugins' ) || ! current_user_can( 'edit_others_posts' ) ) ) {
			$query['author'] = $user_id;
		}

		return $query;
	}

	/**
	 * Insert featured image
	 * @since 1.0.0
	 * Specific to include_image_upload option
	 * @since 1.5.3
	 */
	public function filter_post_id( $my_post_id ) {
		if( $this -> is_uploading_permitted() && ! empty( $options['include_image_upload'] ) ) {
			if( $_POST['topic-image-id'] ) {
				set_post_thumbnail( $my_post_id, absint( $_POST['topic-image-id'] ) );
			}
		}
		return $my_post_id;
	}

	// Filter mime types
	public function custom_upload_mimes( $existing_mimes=array() ) {
		if( current_user_can( 'upload_files' ) ) {
			// If we're an admin, we can upload anything as normal
		} else {
			// Check if we have uploading permission
			if( $this -> is_uploading_permitted() ) {
				$existing_mimes = array(); // Reset list
				$existing_mimes['png'] = 'image/png';
				$existing_mimes['jpg'] = 'image/jpg';
				$existing_mimes['jpeg'] = 'image/jpeg';
				apply_filters( 'ctdb_existing_mimes', $existing_mimes );
			}
		}
		return $existing_mimes;
	}

	// Rename image on upload
	public function rename_upload( $filename ) {
		$rename_upload = apply_filters( 'ctdb_filter_rename_upload', true );
		if( current_user_can( 'upload_files' ) ) {
			// If we're an admin, the file doesn't get renamed
		} else if( $rename_upload ) {
			// Check if we have uploading permission
			if( $this -> is_uploading_permitted() ) {
				$info = pathinfo( $filename );
				$ext  = empty( $info['extension'] ) ? '' : '.' . $info['extension'];
				$name = basename( $filename, $ext );
				// Return renamed file
				return md5( $name ) . $ext;
			}
		}
		return $filename;
	}

	/**
	 * Returns whether current user is permitted to upload images
	 * Checks for posting permissions and ensures user is logged in
	 * @since 1.0.0
	 * Check that upload image option is set
	 * @since 1.1.0
	 * Added enable_media_button setting
	 * @since 1.5.3
	 */
	public function is_uploading_permitted() {
		$options = get_option( 'ctdb_pro_settings' );
		if( is_user_logged_in() && ctdb_is_posting_permitted() && ( ! empty( $options['include_image_upload'] ) || ! empty( $options['enable_media_button'] ) ) ) {
			return true;
		}

		return false;

	}
}
