<?php
/**
 * Widget API: WP_Widget_Recent_Replies class
 *
 * @package WordPress
 * @subpackage Widgets
 * @since 4.4.0
 */

/**
 * Core class used to implement a Recent Replies widget.
 *
 * @since 2.8.0
 *
 * @see WP_Widget
 */
if( ! class_exists( 'WP_Widget_Recent_Replies' ) ) {
	class WP_Widget_Recent_Replies extends WP_Widget {

		/**
		 * Sets up a new Recent Replies widget instance.
		 *
		 * @since 2.8.0
		 * @access public
		 */
		public function __construct() {
			$widget_ops = array(
				'classname' => 'widget_recent_replies',
				'description' => __( 'Your site&#8217;s most recent replies to discussion topics.', 'discussion-board-pro' ),
				'customize_selective_refresh' => true,
			);
			parent::__construct( 'ctdb-widget-profile-list', __( 'Recent Replies', 'discussion-board-pro' ), $widget_ops );
			$this->alt_option_name = 'widget_recent_replies';

			if ( is_active_widget( false, false, $this->id_base ) || is_customize_preview() ) {
				add_action( 'wp_head', array( $this, 'recent_comments_style' ) );
			}
		}

	 	/**
		 * Outputs the default styles for the Recent Comments widget.
		 *
		 * @since 2.8.0
		 * @access public
		 */
		public function recent_comments_style() {
			/**
			 * Filters the Recent Comments default widget styles.
			 *
			 * @since 3.1.0
			 *
			 * @param bool   $active  Whether the widget is active. Default true.
			 * @param string $id_base The widget ID.
			 */
			if ( ! current_theme_supports( 'widgets' ) // Temp hack #14876
				|| ! apply_filters( 'show_recent_comments_widget_style', true, $this->id_base ) )
				return;
			?>
			<style type="text/css">.recentcomments a{display:inline !important;padding:0 !important;margin:0 !important;}</style>
			<?php
		}

		/**
		 * Outputs the content for the current Recent Replies widget instance.
		 *
		 * @since 2.8.0
		 * @access public
		 *
		 * @param array $args     Display arguments including 'before_title', 'after_title',
		 *                        'before_widget', and 'after_widget'.
		 * @param array $instance Settings for the current Recent Comments widget instance.
		 */
		public function widget( $args, $instance ) {
			if ( ! isset( $args['widget_id'] ) )
				$args['widget_id'] = $this->id;

			$output = '';

			$title = ( ! empty( $instance['title'] ) ) ? $instance['title'] : __( 'Recent Replies', 'discussion-board-pro' );

			/** This filter is documented in wp-includes/widgets/class-wp-widget-pages.php */
			$title = apply_filters( 'widget_title', $title, $instance, $this->id_base );

			$number = ( ! empty( $instance['number'] ) ) ? absint( $instance['number'] ) : 5;
			if ( ! $number )
				$number = 5;
			
			$words = ( ! empty( $instance['words'] ) ) ? absint( $instance['words'] ) : 0;
			if ( ! $words )
				$words = 0;
			
			$gravatar = isset( $instance['use_gravatar'] ) ? (bool)( $instance['use_gravatar'] ) : false;

			/**
			 * Filters the arguments for the Recent Comments widget.
			 *
			 * @since 3.4.0
			 *
			 * @see WP_Comment_Query::query() for information on accepted arguments.
			 *
			 * @param array $comment_args An array of arguments used to retrieve the recent comments.
			 */
			$comments = get_comments( apply_filters( 'widget_comments_args', array(
				'number'      	=> $number,
				'status'      	=> 'approve',
				'post_status' 	=> 'publish',
				'post_type'		=> 'discussion-topics'
			) ) );

			$output .= $args['before_widget'];
			if ( $title ) {
				$output .= $args['before_title'] . $title . $args['after_title'];
			}

			$classes = array(
				'recentcomments',
				'ctdb-widget-profile-list'
			);
			if( $gravatar ) {
				$classes[] = 'with-gravatar';
			}
			$show_date = isset( $instance['show_date'] ) ? $instance['show_date'] : false;
			
			$output .= '<ul class="' . join( ' ', $classes ) . '">';
			if ( is_array( $comments ) && $comments ) {
				// Prime cache for associated posts. (Prime post term cache if we need it for permalinks.)
				$post_ids = array_unique( wp_list_pluck( $comments, 'comment_post_ID' ) );
				_prime_post_caches( $post_ids, strpos( get_option( 'permalink_structure' ), '%category%' ), false );

				foreach ( (array) $comments as $comment ) {
					$output .= '<li class="recent-reply">';
					if( $gravatar ) {
						$avatar_size = 48;
						$avatar_size = apply_filters( 'ctdb_recent_replies_avatar_size', $avatar_size );
						$output .= '<div class="ctdb-recent-gravatar">' . get_avatar( $comment, $avatar_size ) . '</div>';
					}
					$output .= '<div class="ctdb-widget-profile-meta">';
					if( $words > 0 ) {
						$output .= '<span class="ctdb-recent-content">' . wp_trim_words( $comment->comment_content, absint( $words ) ) . '</span><br>';
					}
					
					$commenter_url = apply_filters( 'ctdb_widget_filter_comment', $comment );
					$commenter_url = get_comment_author_link( $comment );
					
					$output .= sprintf( _x( '%1$s on %2$s', 'widgets' ),
						'<span class="comment-author-link">' . $commenter_url . '</span>',
						'<a href="' . esc_url( get_comment_link( $comment ) ) . '">' . get_the_title( $comment->comment_post_ID ) . '</a>'
					);
					$output .= '<div class="ctdb-widget-profile-meta-wrapper">';
						if ( $show_date ) {
							$date_format = get_option( 'date_format' );
							$comment_data = new DateTime( $comment->comment_date );
							$comment_data = $comment_data->format( $date_format );
							$output .= '<div class="post-date">' . $comment_data . '</div>';
						}
					$output .= '</div>';
					$output .= '</div>';
					$output .= '</li>';
				}
			}
			$output .= '</ul>';
			$output .= $args['after_widget'];

			echo $output;
		}

		/**
		 * Handles updating settings for the current Recent Comments widget instance.
		 *
		 * @since 2.8.0
		 * @access public
		 *
		 * @param array $new_instance New settings for this instance as input by the user via
		 *                            WP_Widget::form().
		 * @param array $old_instance Old settings for this instance.
		 * @return array Updated settings to save.
		 */
		public function update( $new_instance, $old_instance ) {
			$instance = $old_instance;
			$instance['title'] = sanitize_text_field( $new_instance['title'] );
			$instance['number'] = absint( $new_instance['number'] );
			$instance['words'] = isset( $new_instance['words'] ) ? absint( $new_instance['words'] ) : 0;
			$instance['use_gravatar'] = $new_instance['use_gravatar'];
			$instance['show_date'] = isset( $new_instance['show_date'] ) ? (bool) $new_instance['show_date'] : false;
			return $instance;
		}

		/**
		 * Outputs the settings form for the Recent Comments widget.
		 *
		 * @since 2.8.0
		 * @access public
		 *
		 * @param array $instance Current settings.
		 */
		public function form( $instance ) {
			$title = isset( $instance['title'] ) ? $instance['title'] : '';
			$number = isset( $instance['number'] ) ? absint( $instance['number'] ) : 5;
			$words = isset( $instance['words'] ) ? absint( $instance['words'] ) : 25;
			$gravatar = isset( $instance['use_gravatar'] ) ? (bool)( $instance['use_gravatar'] ) : false;
			$show_date = isset( $instance['show_date'] ) ? (bool) $instance['show_date'] : false;
			?>
			
			<p><label for="<?php echo $this->get_field_id( 'title' ); ?>"><?php _e( 'Title:', 'discussion-board-pro' ); ?></label>
			<input class="widefat" id="<?php echo $this->get_field_id( 'title' ); ?>" name="<?php echo $this->get_field_name( 'title' ); ?>" type="text" value="<?php echo esc_attr( $title ); ?>" /></p>

			<p><label for="<?php echo $this->get_field_id( 'number' ); ?>"><?php _e( 'Number of comments to show:', 'discussion-board-pro' ); ?></label>
			<input class="tiny-text" id="<?php echo $this->get_field_id( 'number' ); ?>" name="<?php echo $this->get_field_name( 'number' ); ?>" type="number" step="1" min="1" value="<?php echo $number; ?>" size="3" /></p>
			
			<p><label for="<?php echo $this->get_field_id( 'words' ); ?>"><?php _e( 'Comment excerpt length:', 'discussion-board-pro' ); ?></label>
			<input class="tiny-text" id="<?php echo $this->get_field_id( 'words' ); ?>" name="<?php echo $this->get_field_name( 'words' ); ?>" type="number" step="1" min="0" value="<?php echo $words; ?>" size="3" /></p>
			
			<p><input class="checkbox" type="checkbox" <?php checked( $gravatar ); ?> id="<?php echo $this->get_field_id( 'use_gravatar' ); ?>" name="<?php echo $this->get_field_name( 'use_gravatar' ); ?>" />
			<label for="<?php echo $this->get_field_id( 'use_gravatar' ); ?>"><?php _e( 'Display gravatar?' ); ?></label></p>
			
			<p><input class="checkbox" type="checkbox"<?php checked( $show_date ); ?> id="<?php echo $this->get_field_id( 'show_date' ); ?>" name="<?php echo $this->get_field_name( 'show_date' ); ?>" />
			<label for="<?php echo $this->get_field_id( 'show_date' ); ?>"><?php _e( 'Display post date?', 'discussion-board-pro' ); ?></label></p>
			<?php
		}

	}
}
