<?php
/**
 * Widget API: WP_Widget_Recent_Topics class
 * @since 1.1.1
 */

/**
 * Core class used to implement a Recent Topics widget.
 *
 * @since 2.8.0
 *
 * @see WP_Widget
 */
class WP_Widget_Recent_Topics extends WP_Widget {

	/**
	 * Sets up a new Recent Topics widget instance.
	 *
	 * @since 2.8.0
	 * @access public
	 */
	public function __construct() {
		$widget_ops = array(
			'classname' => 'widget_recent_topics',
			'description' => __( 'Your site&#8217;s most recent Discussion Topics.', 'discussion-board-pro' ),
			'customize_selective_refresh' => true,
		);
		parent::__construct( 'recent-topics', __( 'Recent Topics', 'discussion-board-pro' ), $widget_ops );
		$this->alt_option_name = 'widget_recent_topics';
	}

	/**
	 * Outputs the content for the current Recent Topics widget instance.
	 *
	 * @since 2.8.0
	 * @access public
	 *
	 * @param array $args     Display arguments including 'before_title', 'after_title',
	 *                        'before_widget', and 'after_widget'.
	 * @param array $instance Settings for the current Recent Posts widget instance.
	 */
	public function widget( $args, $instance ) {
		
		global $post;
		
		if ( ! isset( $args['widget_id'] ) ) {
			$args['widget_id'] = $this->id;
		}

		$title = ( ! empty( $instance['title'] ) ) ? $instance['title'] : __( 'Recent Topics', 'discussion-board-pro' );

		/** This filter is documented in wp-includes/widgets/class-wp-widget-pages.php */
		$title = apply_filters( 'widget_title', $title, $instance, $this->id_base );

		$number = ( ! empty( $instance['number'] ) ) ? absint( $instance['number'] ) : 5;
		if ( ! $number )
			$number = 5;
		$show_date = isset( $instance['show_date'] ) ? $instance['show_date'] : false;
		$show_author = isset( $instance['show_author'] ) ? $instance['show_author'] : false;
		
		$words = ( ! empty( $instance['words'] ) ) ? absint( $instance['words'] ) : 0;
		if ( ! $words )
			$words = 0;
		
		$gravatar = isset( $instance['use_gravatar'] ) ? (bool)( $instance['use_gravatar'] ) : false;

		/**
		 * Filters the arguments for the Recent Posts widget.
		 *
		 * @since 3.4.0
		 *
		 * @see WP_Query::get_posts()
		 *
		 * @param array $args An array of arguments used to retrieve the recent posts.
		 */
		$r = new WP_Query( apply_filters( 'ctdb_widget_topics_args', array(
			'posts_per_page'		=> $number,
			'no_found_rows'			=> true,
			'post_status'         	=> 'publish',
			'ignore_sticky_posts'	=> true,
			'post_type'				=> 'discussion-topics'
		) ) );
		
		$output = '';

		if ($r->have_posts()) : 
			$classes = array(
				'recentcomments',
				'ctdb-widget-profile-list'
			);
			if( $gravatar ) {
				$classes[] = 'with-gravatar';
			}
			$output = $args['before_widget'];
			if ( $title ) {
				$output .= $args['before_title'] . $title . $args['after_title'];
			}
			$output .= '<ul class="' . join( ' ', $classes ) . '">';
				while ( $r->have_posts() ) : $r->the_post();
					$output .= '<li>';
						$author = $post->post_author;
						if( $gravatar ) {
							$avatar_size = 48;
							$avatar_size = apply_filters( 'ctdb_recent_replies_avatar_size', $avatar_size );
							$output .= '<div class="ctdb-recent-gravatar">' . get_avatar( $author, $avatar_size ) . '</div>';
						}
						$output .= '<div class="ctdb-widget-profile-meta">';
							$output .= '<a href="'. get_permalink() . '">' . get_the_title() . '</a>';
							if( $words > 0 ) {
								if( ! empty( $post->post_excerpt ) ) {
									$excerpt = $post->post_excerpt;
								} else {
									$excerpt = strip_shortcodes( $post->post_content );
								}
								if( ! empty( $excerpt ) ) {
									$output .= '<div class="ctdb-recent-content">' . wp_kses_post( wp_trim_words( $excerpt, absint( $words ) ) ) . '</div>';
								}
								
							}
							$output .= '<div class="ctdb-widget-profile-meta-wrapper">';
							
							if( $show_author ) {
								$author_url = esc_url( get_author_posts_url( get_the_author_meta( 'ID' ) ) );

								$output .= '<div><span class="comment-author-link">' . __( 'By', 'discussion-board-pro' ) . ' <a href="' . $author_url . '">' . get_the_author() . '</a></span></div>';
							}
								
							if ( $show_date ) {
								$output .= '<div><span class="post-date">' . get_the_date() . '</span></div>';
							}
							$output .= '</div>';
							
						$output .= '</div>';
					$output .= '</li>';
				endwhile;
			$output .= '</ul>';
			$output .= $args['after_widget'];
			
			echo $output;
			
			// Reset the global $the_post as this query will have stomped on it
			wp_reset_postdata();

		endif;
	}

	/**
	 * Handles updating the settings for the current Recent Topics widget instance.
	 *
	 * @since 2.8.0
	 * @access public
	 *
	 * @param array $new_instance New settings for this instance as input by the user via
	 *                            WP_Widget::form().
	 * @param array $old_instance Old settings for this instance.
	 * @return array Updated settings to save.
	 */
	public function update( $new_instance, $old_instance ) {
		$instance = $old_instance;
		$instance['title'] = sanitize_text_field( $new_instance['title'] );
		$instance['number'] = (int) $new_instance['number'];
		$instance['words'] = isset( $new_instance['words'] ) ? absint( $new_instance['words'] ) : 0;
		$instance['use_gravatar'] = $new_instance['use_gravatar'];
		$instance['show_author'] = isset( $new_instance['show_author'] ) ? (bool) $new_instance['show_author'] : false;
		$instance['show_date'] = isset( $new_instance['show_date'] ) ? (bool) $new_instance['show_date'] : false;
		return $instance;
	}

	/**
	 * Outputs the settings form for the Recent Topics widget.
	 *
	 * @since 2.8.0
	 * @access public
	 *
	 * @param array $instance Current settings.
	 */
	public function form( $instance ) {
		$title     = isset( $instance['title'] ) ? esc_attr( $instance['title'] ) : '';
		$number    = isset( $instance['number'] ) ? absint( $instance['number'] ) : 5;
		$words = isset( $instance['words'] ) ? absint( $instance['words'] ) : 25;
		$gravatar = isset( $instance['use_gravatar'] ) ? (bool)( $instance['use_gravatar'] ) : false;
		$show_author = isset( $instance['show_author'] ) ? (bool) $instance['show_author'] : false;
		$show_date = isset( $instance['show_date'] ) ? (bool) $instance['show_date'] : false;
?>
		<p><label for="<?php echo $this->get_field_id( 'title' ); ?>"><?php _e( 'Title:', 'discussion-board-pro' ); ?></label>
		<input class="widefat" id="<?php echo $this->get_field_id( 'title' ); ?>" name="<?php echo $this->get_field_name( 'title' ); ?>" type="text" value="<?php echo $title; ?>" /></p>

		<p><label for="<?php echo $this->get_field_id( 'number' ); ?>"><?php _e( 'Number of posts to show:', 'discussion-board-pro' ); ?></label>
		<input class="tiny-text" id="<?php echo $this->get_field_id( 'number' ); ?>" name="<?php echo $this->get_field_name( 'number' ); ?>" type="number" step="1" min="1" value="<?php echo $number; ?>" size="3" /></p>
		
		<p><label for="<?php echo $this->get_field_id( 'words' ); ?>"><?php _e( 'Comment excerpt length:', 'discussion-board-pro' ); ?></label>
		<input class="tiny-text" id="<?php echo $this->get_field_id( 'words' ); ?>" name="<?php echo $this->get_field_name( 'words' ); ?>" type="number" step="1" min="0" value="<?php echo $words; ?>" size="3" /></p>
		
		<p><input class="checkbox" type="checkbox" <?php checked( $gravatar ); ?> id="<?php echo $this->get_field_id( 'use_gravatar' ); ?>" name="<?php echo $this->get_field_name( 'use_gravatar' ); ?>" />
		<label for="<?php echo $this->get_field_id( 'use_gravatar' ); ?>"><?php _e( 'Display gravatar?' ); ?></label></p>
		
		<p><input class="checkbox" type="checkbox" <?php checked( $show_author ); ?> id="<?php echo $this->get_field_id( 'show_author' ); ?>" name="<?php echo $this->get_field_name( 'show_author' ); ?>" />
		<label for="<?php echo $this->get_field_id( 'show_author' ); ?>"><?php _e( 'Display author?' ); ?></label></p>

		<p><input class="checkbox" type="checkbox"<?php checked( $show_date ); ?> id="<?php echo $this->get_field_id( 'show_date' ); ?>" name="<?php echo $this->get_field_name( 'show_date' ); ?>" />
		<label for="<?php echo $this->get_field_id( 'show_date' ); ?>"><?php _e( 'Display post date?', 'discussion-board-pro' ); ?></label></p>
<?php
	}
}
